@@ -20,13 +20,30 @@
 
 #include <lineak/lconfig.h>
 #include <lineak/lineak_core_functions.h>
+
+
 using namespace lineak_core_functions;
+using namespace std;
+
+
 #include "xosdctrl.h"
 //#include "lineak_core_functions.h"
 //float xosdCtrl::MaxAudio = 25700.00;
 //const float xosdCtrl::MUTE = -100.0000;
 
 xosdCtrl::xosdCtrl(LConfig& config) : displayCtrl(config) {
+  
+  Lines = 2;
+  
+
+  if (sem_init(&enter_sem, 0,1) != 0)
+     error("enter_sem: Semaphore initialization failed");
+  if (sem_init(&exit_sem,0,1) != 0)
+     error("exit_sem: Semaphore initialization failed");
+
+  init(config);
+
+/*
   ScrollAge = true;
   create(2);
   setFont(config["Display_font"]);
@@ -67,16 +84,22 @@ xosdCtrl::xosdCtrl(LConfig& config) : displayCtrl(config) {
   vmsg("");
   
   Forcewait=false;
-  ScreenLine = 0;
+  ScreenLine = 0; */
   /* Not really needed, but at least we aren't throwing around an unknown value */
-  old_age.tv_sec=0;
+  /*old_age.tv_sec=0;
 
   if (ScrollAge)
-    gettimeofday(&old_age,0);
+    gettimeofday(&old_age,0); */
 }
 
-xosdCtrl::xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_align align, int hoffset, int voffset, int soffset, int lines) {
-  ScrollAge = true;
+xosdCtrl::xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_align align, int hoffset, int voffset, int soffset, int lines): Lines(lines) {
+  //ScrollAge = true;
+
+  if (sem_init(&enter_sem, 0,1) != 0)
+     error("enter_sem: Semaphore initialization failed");
+  if (sem_init(&exit_sem,0,1) != 0)
+     error("exit_sem: Semaphore initialization failed");
+
   create(lines);
   setFont(font);
   setColor(color);
@@ -86,8 +109,8 @@ xosdCtrl::xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_alig
   setHorizontalOffset(hoffset);
   setVerticalOffset(voffset);
   setShadowOffset(soffset);
-  Forcewait=false;
-  ScreenLine = 0;
+  //Forcewait=false;
+  //ScreenLine = 0;
   /* Not really needed, but at least we aren't throwing around an unknown value */
   old_age.tv_sec=0;
 
@@ -96,8 +119,25 @@ xosdCtrl::xosdCtrl(string font, string color, int delay, xosd_pos pos, xosd_alig
 }
 
 xosdCtrl::xosdCtrl() : displayCtrl() {
-/* Set my defaults */
-  ScrollAge = true;
+
+  if (sem_init(&enter_sem, 0,1) != 0)
+     error("enter_sem: Semaphore initialization failed");
+  if (sem_init(&exit_sem,0,1) != 0)
+     error("exit_sem: Semaphore initialization failed");
+  /* Set my defaults */
+  //ScrollAge = true;
+  Lines = 2;
+  Font = DISPLAY_FONT;
+  Color = DISPLAY_COLOR;
+  Timeout = atoi(DISPLAY_TIMEOUT);
+  Pos = XOSD_bottom;
+  Align = XOSD_center;
+  HorizontalOffset = atoi(DISPLAY_HOFFSET);
+  VerticalOffset = atoi(DISPLAY_VOFFSET);
+  ShadowOffset = atoi(DISPLAY_SOFFSET);
+  init();
+
+/*
   create(2);
   setFont(DISPLAY_FONT);
   setColor(DISPLAY_COLOR);
@@ -109,18 +149,31 @@ xosdCtrl::xosdCtrl() : displayCtrl() {
   setShadowOffset(atoi(DISPLAY_SOFFSET));
   Forcewait=false;
   ScreenLine = 0;
-  /* Not really needed, but at least we aren't throwing around an unknown value */
+
+  // Not really needed, but at least we aren't throwing around an unknown value //
   old_age.tv_sec=0;
 
   if (ScrollAge)
     gettimeofday(&old_age,0);
-
+*/
 }
 xosdCtrl::~xosdCtrl(){
+  /** Destroy the semaphores */
+  sem_destroy(&enter_sem);
+  sem_destroy(&exit_sem);
 }
 void xosdCtrl::create(int lines) {
-	Lines=lines;
-	osd = xosd_create(Lines);
+    // Set some variables.
+    ScrollAge=true;
+    Forcewait=false;
+    ScreenLine = 0;
+    Lines=lines;
+
+    sem_wait(&enter_sem);
+    // Create the xosd object
+    osd = xosd_create(Lines);
+    sem_post(&enter_sem);
+
     /** This can happen in the situations where LANG is an asian character set */
     if (!osd) {
       cerr << "Error initializing osd: " << xosd_error << endl;
@@ -132,12 +185,18 @@ void xosdCtrl::create(int lines) {
 //      }
     }
    /* Not really needed, but at least we aren't throwing around an unknown value */
-    old_age.tv_sec=0;
+   /* old_age.tv_sec=0;
 
     if (ScrollAge)
     	gettimeofday(&old_age,0);
+*/
+
 }
 void xosdCtrl::init(LConfig& config){
+   
+   vmsg("Initializing xosdCtrl");
+   
+   //ScrollAge=true;
    create(Lines);
 
    setFont(config["Display_font"]);
@@ -159,21 +218,37 @@ void xosdCtrl::init(LConfig& config){
       setAlign((xosd_align)XOSD_right);
 
    setTimeout(atoi(config.getValue("Display_timeout").c_str()));
-   //  setPos((xosd_pos)config.getIntValue("XOSD_pos"));
-   //  setAlign((xosd_align)config.getIntValue("XOSD_align"));
    setHorizontalOffset(atoi(config.getValue("Display_hoffset").c_str()));
    setVerticalOffset(atoi(config.getValue("Display_voffset").c_str()));
    setShadowOffset(atoi(config.getValue("Display_soffset").c_str()));
 
+   //config.print(cout);
+   vmsg("Our parameters!");
+   vmsg(config["Display_font"]);
+   vmsg(config["Display_color"]);
+   vmsg(config["Display_pos"]);
+   vmsg(config["Display_align"]);
+   vmsg(config.getValue("Display_timeout"));
+   vmsg(config.getValue("Display_hoffset"));
+   vmsg(config.getValue("Display_voffset"));
+   vmsg(config.getValue("Display_soffset"));
+   vmsg("");
+
+   //Forcewait=false;
+   //ScreenLine = 0;
    /* Not really needed, but at least we aren't throwing around an unknown value */
     old_age.tv_sec=0;
 
     if (ScrollAge)
     	gettimeofday(&old_age,0);
-
+    
+    vmsg("Finished Initializing");
 }
 void xosdCtrl::init(){
-/* Set my defaults */
+
+    vmsg("Initializing default xosdCtrl");
+
+    /* Set my defaults */
     create(Lines);
     setFont(Font);
     setColor(Color);
@@ -183,23 +258,42 @@ void xosdCtrl::init(){
     setHorizontalOffset(HorizontalOffset);
     setVerticalOffset(VerticalOffset);
     setShadowOffset(ShadowOffset);
+
+//     vmsg("Our parameters!");
+//     vmsg(Font);
+//     vmsg(Color);
+//     vmsg(string(Pos));
+//     vmsg(Align);
+//     vmsg(Timeout);
+//     vmsg(HorizontalOffset);
+//     vmsg(VerticalOffset);
+//     vmsg(ShadowOffset);
+//     vmsg("");
+
+    //Forcewait=false;
+    //ScreenLine = 0;
+
    /* Not really needed, but at least we aren't throwing around an unknown value */
     old_age.tv_sec=0;
 
     if (ScrollAge)
     	gettimeofday(&old_age,0);
+
+    vmsg("Finished Default Initialization");
 }
 
 void xosdCtrl::cleanup() {
    scroll();
-   msg("xosdCtrl::cleanup() -- Cleaning up xosd");
+   vmsg("xosdCtrl::cleanup() -- Cleaning up xosd");
    if (osd != NULL) {
-        msg("xosdCtrl::cleanup() -- xosd_destroy");
+        vmsg("xosdCtrl::cleanup() -- xosd_destroy");
         /** For some reason, xosd_destroy hangs sometimes. */
+        sem_wait(&enter_sem);
   	xosd_destroy (osd);
+        sem_post(&enter_sem);
     	osd = NULL;
     }
-   msg("xosdCtrl::cleanup() -- done cleanup");
+   vmsg("xosdCtrl::cleanup() -- done cleanup");
 }
 
 void xosdCtrl::agenscroll(void) {
@@ -235,9 +329,12 @@ void xosdCtrl::volume(float ivolume) {
   float volcalc;
   scroll();
      if (ivolume == MUTE) {
-        if (osd)
+        if (osd) {
+	   sem_wait(&enter_sem);
            xosd_display (osd, ScreenLine++, XOSD_string, "Volume Muted");
+           sem_post(&enter_sem);
 				//agenscroll();
+        }
      }
      else {
 		volcalc = (float)ivolume;
@@ -245,9 +342,12 @@ void xosdCtrl::volume(float ivolume) {
 		volcalc *= 100.00;
 		int volume = (int) volcalc;
 		if (osd) {
+                        sem_wait(&enter_sem);
 			xosd_display (osd, 0, XOSD_string, "Volume");
 			xosd_display (osd, 1, XOSD_percentage, volume);
-		}
+                        sem_post(&enter_sem);
+		} else
+			error("Could not display volume level. osd is not defined.");
      }
 
 }
@@ -255,7 +355,9 @@ void xosdCtrl::volume(float ivolume) {
 void xosdCtrl::show(string command) {
 	if (osd) {
 		scroll();
+                sem_wait(&enter_sem);
 	  	xosd_display (osd, ScreenLine++, XOSD_string, command.c_str());
+                sem_post(&enter_sem);
 	}
 }
 
@@ -271,11 +373,15 @@ void xosdCtrl::setFont( const string& _newVal){
 		//strcpy(font, Font.c_str());
 		//if (xosd_set_font (osd, font) == -1)
 		// If you cannot set the font, fall back to the default.
+                sem_wait(&enter_sem);
 		if (xosd_set_font (osd, Font.c_str()) == -1) {
-		   error("xosd_set_font: has failed");
+		   error("xosd_set_font has failed: " + Font);
 		   Font = DISPLAY_FONT;
-		   xosd_set_font (osd, DISPLAY_FONT);
+		   error(string("Setting default font: ") + DISPLAY_FONT);
+		   if (xosd_set_font (osd, DISPLAY_FONT) == -1)
+			   error("Setting default font has failed");
 		}
+                sem_post(&enter_sem);
 		if (!osd) {
 		   error("osd has failed to initialize and is now invalid.");
 		   osd = NULL;
@@ -294,7 +400,9 @@ void xosdCtrl::setColor( const string& _newVal){
 		string tmp = '#' + Color;
 		char *color =  (char *)malloc((strlen(tmp.c_str())+1)*sizeof(char));
 		strcpy(color, tmp.c_str());
+		sem_wait(&enter_sem);
 		xosd_set_colour (osd, color);
+		sem_post(&enter_sem);
 		free(color);
 	}
 }
@@ -305,8 +413,11 @@ const int& xosdCtrl::getTimeout(){
 /** Write property of int delay. */
 void xosdCtrl::setTimeout( const int& _newVal){
 	Timeout = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_timeout (osd, Timeout);
+		sem_post(&enter_sem);
+        }
 }
 /** Read property of int Forcewait. */
 const bool& xosdCtrl::getForcewait(){
@@ -323,8 +434,11 @@ const xosd_pos& xosdCtrl::getPos(){
 /** Write property of xosd_pos Pos. */
 void xosdCtrl::setPos( const xosd_pos& _newVal){
 	Pos = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_pos (osd, Pos);
+		sem_post(&enter_sem);
+	}
 }
 /** Read property of int Offset. */
 const int& xosdCtrl::getHorizontalOffset(){
@@ -333,8 +447,11 @@ const int& xosdCtrl::getHorizontalOffset(){
 /** Write property of int Offset. */
 void xosdCtrl::setHorizontalOffset( const int& _newVal){
 	HorizontalOffset = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_horizontal_offset (osd, HorizontalOffset);
+		sem_post(&enter_sem);
+	}
 }
 /** Read property of int Offset. */
 const int& xosdCtrl::getVerticalOffset(){
@@ -343,8 +460,11 @@ const int& xosdCtrl::getVerticalOffset(){
 /** Write property of int Offset. */
 void xosdCtrl::setVerticalOffset( const int& _newVal){
 	VerticalOffset = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_vertical_offset (osd, VerticalOffset);
+		sem_post(&enter_sem);
+	}
 }
 /** Read property of int Shadow. */
 const int& xosdCtrl::getShadowOffset(){
@@ -353,8 +473,11 @@ const int& xosdCtrl::getShadowOffset(){
 /** Write property of int Shadow. */
 void xosdCtrl::setShadowOffset( const int& _newVal){
 	ShadowOffset = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_shadow_offset (osd, ShadowOffset);
+		sem_post(&enter_sem);
+        }
 }
 /** Read property of int ScrollAge. */
 const bool& xosdCtrl::getScrollAge(){
@@ -389,28 +512,41 @@ const xosd_align& xosdCtrl::getAlign(){
 /** Write property of xosd_align Align. */
 void xosdCtrl::setAlign( const xosd_align& _newVal){
 	Align = _newVal;
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_set_align (osd, Align);
+		sem_post(&enter_sem);
+	}
 }
 void xosdCtrl::show() {
-	if (osd)
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_show (osd);
+		sem_post(&enter_sem);
+	}
 }
 void xosdCtrl::hide() {
-	if (osd) 
+	if (osd) {
+		sem_wait(&enter_sem);
 		xosd_hide (osd);
+		sem_post(&enter_sem);
+	}
 }
 /** Read property of xosd_align Align. */
 /** Scroll the display lines number of lines */
 void xosdCtrl::scroll(int lines){
 	if (osd) {
 		if (lines != 0) {
+			sem_wait(&enter_sem);
 			xosd_scroll(osd,lines);
+			sem_post(&enter_sem);
                 }
 		else {
+			sem_wait(&enter_sem);
                         int nlines = 0;
                         nlines = xosd_get_number_lines(osd);
 			xosd_scroll(osd,nlines);
+			sem_post(&enter_sem);
                 }
 	}
 }
